/* Copyright INMOS Limited 1988,1990 */

/* CMSIDENTIFIER */
static char *CMS_Id = "PRODUCT:ITEM.VARIANT-TYPE;0(DATE)";

#include <stdio.h>
#include "server.h"
#include "iserver.h"
#include "misc.h"
#include "boot.h"
#include "linkops.h"
#include "opserror.h"

/*
 * Boot  -  copy the boot file to the link
 */

void     Boot()
{
   FILE            *fd;
   char            buffer[BOOT_BUFFER_LENGTH];
   int             length = 0;
   long            size = 0;
   static char     Flicks[] = "|/-\\|/-\\";
   int             flick = 0;
   unsigned char   ops_res;

   /* Open the boot file */
   fd = fopen(BootFileName, "rb");
   if (fd == NULL) {
      sprintf(ErrMsg,"Boot: cannot find file \"%s\"", BootFileName);
      close_server(MISC_EXIT, ErrMsg);
   }

   infomsg("Booting root transputer...");

   /* Read the file in large chunks */
   while ((length = fread(buffer, 1, BOOT_BUFFER_LENGTH, fd)) > 0) {
      dbgmsg("%d", length);

      ops_res = OPS_BootWrite(ConnId, buffer, (unsigned long) length);

      if (ops_res != STATUS_NOERROR) {
         sprintf(ErrMsg,"Boot: write failed after %d bytes because:\n   %s\n",
                           size, lnkops_errorstring);
         close_server(MISC_EXIT, ErrMsg);
      }
                           
      size += length;
      
      if (isatty(1)) {
         infomsg("%c\b", Flicks[flick]);
         if (++flick == 8)       
            flick = 0;
      }
   }

   if (isatty(1))
      infomsg("\r                           \r");
   else
      infomsg("ok\n");
   
   fclose(fd);
   
   dbgmsg("booted %ld bytes", size);
}

#if PSERVER_EXTENSION

#if __STDC__
void    error (const char *str, ...)
#else
void    error (str, a1, a2, a3)
char    *str;
char    *a1, *a2, *a3;
#endif
{
#if __STDC__
#   include <stdarg.h>
    va_list va;
    va_start (va, str);
#endif
    fprintf (stderr, "pserver: ");
#if __STDC__
    vfprintf (stderr, str, va);
    va_end (va);
#else
    fprintf (stderr, str, a1, a2, a3);
#endif
    fprintf (stderr, "\n");
}

/*
 *
 *      bool route (char *rootroute)
 *
 *  Setup specified route by booting intermediate processors with little
 *  message-passing program.
 *
 */

#if __STDC__
bool  route(char *rootroute)
#else
bool  route(rootroute)
char *rootroute;
#endif
{
    static  char    booter[] = {
        0x8f, 0xd0, 0x27, 0x2f, 0x2f, 0x2f, 0x2f, 0x28,
        0x64, 0x40, 0x23, 0xfc, 0xd0, 0xd0, 0x21, 0x40,
        0x23, 0xf3, 0xd8, 0x24, 0xf2, 0x21, 0xf8, 0x24,
        0xf2, 0x21, 0xfc, 0x24, 0xf2, 0x27, 0x2f, 0x2f,
        0x2f, 0x2f, 0x2f, 0x6d, 0x44, 0xe0, 0x24, 0xf2,
        0x27, 0x2f, 0x2f, 0x2f, 0x2f, 0x2f, 0x6d, 0x48,
        0xe0, 0x40, 0x25, 0xf4, 0x22, 0xf9, 0x25, 0xf7,
        0x40, 0xd2, 0x48, 0xd3, 0x72, 0x24, 0xf2, 0xfa,
        0x21, 0xf2, 0x12, 0x4a, 0x22, 0xf1, 0x78, 0x41,
        0xfe, 0x28, 0x4f, 0xd6, 0x16, 0x23, 0x4f, 0x21,
        0xfb, 0xd7, 0x77, 0x41, 0x24, 0xfa, 0x65, 0x47,
        0x21, 0xfb, 0x23, 0x43, 0x21, 0xfb, 0x76, 0x24,
        0xfa, 0x44, 0x78, 0xfa, 0xd4, 0x12, 0x74, 0x44,
        0xf7, 0x72, 0x27, 0x2f, 0x2f, 0x2f, 0x2f, 0x2f,
        0x6f, 0xc0, 0xaf, 0x21, 0x4a, 0x21, 0xfb, 0x76,
        0x25, 0xf2, 0x23, 0xff, 0x41, 0x25, 0xf2, 0x44,
        0xf8, 0xd2, 0x13, 0x74, 0x44, 0xf7, 0x72, 0x78,
        0x21, 0x40, 0x25, 0xf2, 0x73, 0xf7, 0x72, 0xf6
     };
#if 1
#   include "msgpass.h"
#else
    static  char    msgpasser[] = {
        0x00, 0x00, 0x00, 0x80, 0x73, 0x00, 0x00, 0x00,
        0x40, 0xd5, 0x15, 0x74, 0x41, 0xf7, 0x75, 0xc5,
        0x24, 0xa0, 0x40, 0xd5, 0x15, 0x74, 0x41, 0xf7,
        0x75, 0x44, 0x25, 0xf2, 0x24, 0xf2, 0xfa, 0xd3,
        0x75, 0x24, 0xf2, 0xfa, 0xd5, 0x42, 0xd1, 0x64,
        0x40, 0x21, 0xfb, 0xd0, 0x11, 0x60, 0x46, 0x10,
        0xd7, 0x77, 0xfa, 0xd6, 0x76, 0x22, 0x40, 0x24,
        0xfa, 0x4b, 0x76, 0xfd, 0x12, 0x74, 0x41, 0xf7,
        0x75, 0x72, 0xfe, 0x60, 0x07, 0x10, 0xf3, 0x11,
        0x72, 0x41, 0xf7, 0x77, 0x71, 0xfe, 0x60, 0x07,
        0x15, 0xf3, 0x75, 0x44, 0x25, 0xf2, 0x24, 0xf2,
        0xfa, 0xd3, 0x75, 0x24, 0xf2, 0xfa, 0xd5, 0x76,
        0x41, 0x25, 0xf2, 0xd6, 0x77, 0x75, 0x76, 0xfb,
        0x11, 0x73, 0x41, 0xf7, 0x75, 0x24, 0xf2, 0xff,
        0x75, 0x27, 0x43, 0xff, 0x72, 0x75, 0x27, 0x43,
        0xfb, 0x65, 0x0a
    };
#endif
    int res;

    /* Try to establish path to alternative root processor, if any.     */
    if (rootroute && rootroute[0]) {
        /* Boot root processor.                                         */
        res = WriteLink (ConnId, booter, sizeof(booter), 5);
        if (res < 0)
            error ("error booting router: %d", res);
        else if (res != sizeof(booter))
            error ("timeout booting router after 0x%x bytes", res);
        else {
            char    ack;

            /* Expect to get byte 1 back from booter.                   */
            res = ReadLink (ConnId, &ack, 1, 5);
            if (res < 0)
                error ("error booting router: %d", res);
            else if (res != 1)
                error ("router protocol error: no ack");
            else if (ack != 1)
                error ("router protocol error: ack %d instead of 1", ack);
            else {

                /* Send message passer code to root processor.          */
                res = WriteLink (ConnId, msgpasser, sizeof(msgpasser), 5);
                if (res < 0)
                    error ("error booting router: %d", res);
                else if (res != sizeof(msgpasser))
                    error ("timeout booting router after 0x%x bytes", sizeof(booter)+res);
                else {
                    char    *r;

                    /* Now establish path.                              */
                    for (r=rootroute; *r; r++) {
                        char    linkspec;
                        static  int     resetbit = -1;
			static char *resetmode = NULL;

                        /* Find out reset mode once.                    */
                        if (resetbit == -1) {
                            if (resetmode || (resetmode = getenv ("TRESET")))
                                if (! strncmp (resetmode, "subsystem", strlen(resetmode)))
                                    resetbit = 0x40;
                                 else if (! strncmp (resetmode, "parsytec", strlen(resetmode)))
                                    resetbit = 0x80;
                                else if (! strncmp (resetmode, "none", strlen(resetmode)))
                                    resetbit = 0;
                                else {
                                    fprintf (stderr, "unknown TRESET reset mode: '%s'\n", resetmode);
                                    resetbit = 0;
                                }
                            else
                                resetbit = 0;
                        }

                        /* Indicate if this will be last link spec.     */
                        if (* (r + 1) == '\0') {
                            linkspec = 5;
                            res = WriteLink (ConnId, &linkspec, 1, 5);
                            if (res != 1) {
                                if (res < 0)
                                    error ("error setting up route: %d", res);
                                else
                                    error ("route %s failed after %d processors",
                                                        rootroute, r-rootroute);
                                break;
                            }
                        }

                        /* Send link indicator (0..3).                  */
                        linkspec = (*r - '0') | resetbit;
                        res = WriteLink (ConnId, &linkspec, 1, 5);
                        if (res != 1) {
                            if (res < 0)
                                error ("error setting up route: %d", res);
                            else
                                error ("route %s failed after %d processors",
                                                        rootroute, r-rootroute);
                            break;
                        }
                    }

                    /* Return success if entire path set up.            */
                    return *r == '\0';
                }
            }
        }

        /* If we didn't return success earlier, we didn't make it.      */
        return FALSE;
    }

    /* No route so succeeded.                                           */
    return TRUE;
}
#endif

